/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.backup.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.JarOutputStream;
import net.sf.hibernate.Session;
import org.apache.commons.logging.Log;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.backup.BackupService;
import org.exoplatform.services.backup.ExportLogger;
import org.exoplatform.services.backup.ImportLogger;
import org.exoplatform.services.backup.ImporterExporter;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.Query;
import org.exoplatform.services.organization.User;

/**
 * May 27, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $ID$
 **/
public class BackupServiceImpl implements BackupService {
	private String directory_ ;
	private String userDataDirectory_ ;
	private String serviceDataDirectory_ ;
	private Log log_ ;
	private ExportLogger exportLogger_ ;
	private ImportLogger importLogger_ ;
	private OrganizationService orgService_ ;
	private HibernateService hservice_ ;
	
	public BackupServiceImpl(LogService lservice, 
			                     OrganizationService orgService,
													 HibernateService hservice) {
		orgService_ = orgService ;
		hservice_ = hservice ;
		log_ = lservice.getLog("org.exoplatform.services.exporter") ;
		exportLogger_ = new ExportLogger() ;
		importLogger_ = new ImportLogger() ;
		try {
			setDirectory(System.getProperty("java.io.tmpdir"));
		} catch (Exception ex){
			log_.error("Cannot set the import/export directory for backup service", ex) ;
		}
	}
	
	public List getImportersExporters() {
    PortalContainer pcontainer = PortalContainer.getInstance() ;
    List list  = pcontainer.getComponentInstancesOfType(ImporterExporter.class) ;
		return list ;
	}

	public String getDirectory() { return directory_ ;}
	public void   setDirectory(String s) throws Exception { 
		File dir = new File(s) ;
		if (!dir.exists()) {
			dir.mkdirs() ;
		}
		if(dir.isDirectory()) {
			directory_ = dir.getPath();
			userDataDirectory_ = directory_ +  File.separator + "user-data" ;
			dir = new File(userDataDirectory_) ;
			if(!dir.exists())dir.mkdir() ;
			serviceDataDirectory_ = directory_ +  File.separator + "service-data" ;
			dir = new File(serviceDataDirectory_) ;
			if(!dir.exists())dir.mkdir() ;
		} else {
			throw new Exception(s + " is a file. Cannot create an export directory for backup service") ;
		}
	}
	
	public String getUserDataDirectory() { return userDataDirectory_ ; }
	public String getServiceDataDirectory() { return serviceDataDirectory_ ; }
	
	synchronized public void addImporterExporter(ImporterExporter exporter) {
		//importersExporters_.add(exporter) ;
	}
	
	public ImporterExporter getImporterExporter(String name) {
		List importersExporters = getImportersExporters() ;
		ImporterExporter exporter = null ;
	  for(int j = 0; j < importersExporters.size() ; j++) {
			exporter = (ImporterExporter) importersExporters.get(j) ; 
			if(name.equals(exporter.getName())) {
				return exporter ;
			}
		}
	  return null ;
	}
	
	
	public ExportLogger getExportLogger() { return exportLogger_ ; }
	
	public ImportLogger getImportLogger() { return importLogger_ ; }
	
	public void exportUserData()  throws Exception {
		Collection users = orgService_.getUserPageList(20).getAll() ;
		Iterator i = users.iterator();
		while(i.hasNext()) {
			User user = (User) i.next() ;
			String userName = user.getUserName() ;
		  exportUserData(userName) ;
		}
	}
	
	public void exportUserData(String userName)  throws Exception {
		Session session = null ;
		String fileName = userDataDirectory_ + File.separator + userName + ".jar" ;
	  JarOutputStream out = new JarOutputStream(new FileOutputStream(fileName)) ;
	  List importersExporters = getImportersExporters() ;
		try {
			session = hservice_.openSession() ;
			for(int j = 0; j < importersExporters.size() ; j++) {
				ImporterExporter exporter = (ImporterExporter) importersExporters.get(j) ;
				JarEntry entry = new JarEntry(exporter.getName()+ ".xml") ;
				out.putNextEntry(entry) ;
				exporter.exportUserData(userName, session, out, entry) ;
				out.closeEntry() ;
			}
			session.flush() ;
			hservice_.closeSession() ;
			exportLogger_.log(userName) ;
		} catch (Exception ex) {
			exportLogger_.log(userName, ex) ;
			if(session != null) session.close() ;
			ex.printStackTrace() ;
		}
		out.close() ;
	}
	
	public void importUserData() throws Exception   {
		File dir = new File(userDataDirectory_) ;
		String[] file = dir.list() ;
		for(int i = 0 ; i < file.length; i++) {
			int idx = file[i].indexOf('.') ;
	    String userName = file[i].substring(0, idx) ;
	    importUserData(userName) ;
		}
	}
	
	public void importUserData(String userName) throws Exception  {
		Session session = null ;
		String fileName = userDataDirectory_ + File.separator + userName + ".jar" ;
	  JarFile jar  = new JarFile(new File(fileName)) ;
	  List importersExporters = getImportersExporters() ;
		try {
			session = hservice_.openSession() ;
			for(int j = 0; j < importersExporters.size() ; j++) {
				ImporterExporter exporter = (ImporterExporter) importersExporters.get(j) ;
				String entryName = exporter.getName()+ ".xml" ;
				JarEntry entry = jar.getJarEntry(entryName) ;
				InputStream in = jar.getInputStream(entry) ;
				exporter.importUserData(userName, session, in, entry) ;
				in.close() ;
			}
			session.flush() ;
      hservice_.closeSession() ;
			importLogger_.log(userName) ;
		} catch (Exception ex) {
			importLogger_.log(userName, ex) ;
			ex.printStackTrace() ;
		}
	  jar.close() ;
	}
	
	public void exportServiceData()  throws Exception {
		List importersExporters = getImportersExporters() ;
	  for(int j = 0; j < importersExporters.size() ; j++) {
			ImporterExporter exporter = (ImporterExporter) importersExporters.get(j) ;
			exportServiceData(exporter.getName()) ;
		}
	}
	
	public void exportServiceData(String serviceName)  throws Exception {
		Session session = null ;
		ImporterExporter exporter = getImporterExporter(serviceName) ;
		String fileName = serviceDataDirectory_ + File.separator + serviceName + ".jar" ;
		JarOutputStream out = new JarOutputStream(new FileOutputStream(fileName)) ;
		try {
			session = hservice_.openSession() ;
			exporter.exportServiceData(session, out) ;
      out.close() ;
			session.flush() ;
			hservice_.closeSession() ;
			exportLogger_.log(serviceName) ;
		} catch (Exception ex) {
			exportLogger_.log(serviceName, ex) ;
			ex.printStackTrace() ;
    }
	}
	
	public void importServiceData()  throws Exception {
		File dir = new File(serviceDataDirectory_) ;
		String[] file = dir.list() ;
		for(int i = 0 ; i < file.length; i++) {
			int idx = file[i].indexOf('.') ;
	    String serviceName = file[i].substring(0, idx) ;
	    importServiceData(serviceName) ;
		}
	}
	
	public void importServiceData(String serviceName)  throws Exception {
    ImporterExporter exporter = getImporterExporter(serviceName) ;
    if(exporter == null) return ;
		Session session = null ;
		String fileName = serviceDataDirectory_ + File.separator + serviceName + ".jar" ;
	  JarFile jar  = new JarFile(new File(fileName)) ;
		try {
			session = hservice_.openSession() ;
		  exporter.importServiceData(session, jar) ;
			session.flush() ;
			session.close() ;
			importLogger_.log(serviceName) ;
		} catch (Exception ex) {
			importLogger_.log(serviceName, ex) ;
			if(session != null) session.close() ;
			ex.printStackTrace() ;
		}
	  jar.close() ;
	}
}