package org.exoplatform.test;

import java.util.*;
import java.lang.reflect.*;
import junit.framework.TestCase;
/**
 * Apr 22, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: TestBenchMark.java,v 1.1 2004/10/11 23:27:27 tuan08 Exp $
 **/
public class TestBenchMark extends TestCase {
	
  public void testBenchMark() throws Exception {
  	for(int i = 1 ; i <= 0 ; i++) {
  		int loop =  i * 5000000 ;
  		singleInstruction(loop) ;
  		emptyMethodCall(loop) ;
  		directCall(loop) ;
  		directSynchronizedCall(loop) ;
  		newObject(loop) ;
  		reflectionCall("setter", loop) ;
  		reflectionCall("setterSynchronized", loop) ;
  		reflectionMethodCall("setter", loop) ;
  		reflectionMethodCall("setterSynchronized", loop) ;
  		reflectionMethodCallCache("setter", loop) ;
  		reflectionMethodCallCache("setterSynchronized", loop) ;
  		System.out.println("===========================================================") ;
    }
  }
  
  private void singleInstruction(int numOfCall) {
		long start = System.currentTimeMillis();
		for (int i = 0; i < numOfCall; i++) {
			int a  = i ;
		}
		long end = System.currentTimeMillis();
		printInfo("singleInstruction()", numOfCall, start, end);
	}
	
  private void emptyMethodCall(int numOfCall) {
		Dummy dummy = new Dummy();
    long start = System.currentTimeMillis();
    for (int i = 0; i < numOfCall; i++) {
			dummy.empty();
    }
    long end = System.currentTimeMillis();
    printInfo("emptyMethodCall()", numOfCall, start, end);
    }
  
	private void directCall(int numOfCall) {
		Dummy dummy = new Dummy();
	  long start = System.currentTimeMillis() ;
		for (int i = 0; i < numOfCall; i++) {
			dummy.setter("dummy");
		}
	  long end = System.currentTimeMillis() ;
	  printInfo("directCall()", numOfCall, start, end) ; 
	}
	
  private void newObject(int numOfCall) {
    long start = System.currentTimeMillis() ;
    for (int i = 0; i < numOfCall; i++) {
    	Dummy dummy = new Dummy();
    }
    long end = System.currentTimeMillis() ;
    printInfo("newObject()", numOfCall, start, end) ; 
  }
    
  private void directSynchronizedCall(int numOfCall) {
    Dummy dummy = new Dummy();
    long start = System.currentTimeMillis() ;
    for (int i = 0; i < numOfCall; i++) {
      dummy.setterSynchronized("dummy");
    }
    long end = System.currentTimeMillis() ;
    printInfo("directSynchronizedCall()", numOfCall, start, end) ; 
  }
   
	private void reflectionCall(String methodName, int numOfCall) throws Exception {
		Dummy dummy = new Dummy();
	  long start = System.currentTimeMillis() ;
	  Class[] types = {String.class} ;
		Object[] args = {"dummy"} ;
		for (int i = 0; i < numOfCall; i++) {
			Method method = dummy.getClass().getMethod(methodName, types);
			method.invoke(dummy, args);
		}
	  long end = System.currentTimeMillis() ;
	  printInfo("reflectionCall(), method: " + methodName, numOfCall, start, end) ; 
	}
    
	private void reflectionMethodCall(String methodName, int numOfCall) throws Exception {
		Dummy dummy = new Dummy();
		Method method = dummy.getClass().getMethod(methodName, new Class[]{String.class});
	  Class[] types = {String.class} ;
		Object[] args = {"dummy"} ;
	  long start = System.currentTimeMillis() ;
		for (int i = 0; i < numOfCall; i++) {
			method.invoke(dummy, args);
		}
	  long end = System.currentTimeMillis() ;
	  printInfo("reflectionMethodCall(), method: " + methodName, numOfCall, start, end) ; 
	}
    
	private void reflectionMethodCallCache(String methodName, int numOfCall) throws Exception {
		Dummy dummy = new Dummy();
		Map map = new HashMap();
		Method method = dummy.getClass().getMethod(methodName, new Class[]{String.class});
		map.put(methodName, method);
		Object[] args = {"dummy"} ;
	  long start = System.currentTimeMillis() ;
		for (int i = 0; i < numOfCall; i++) {
			Method m = (Method) map.get(methodName);
			m.invoke(dummy, args);
		}
	  long end = System.currentTimeMillis() ;
	  printInfo("reflectionCallMethodCache(), method: " + methodName, numOfCall, start, end) ; 
	}
  
  private void printInfo(String method, int numOfCall, long start, long end) {
  	System.out.println("Call method " + method + " " + numOfCall + " times in " + (end - start) +" ms") ;
  }
}
